#! /usr/bin/env python

import sys
import os
import json
import random
import argparse
import itertools
import collections
from archipelago import profile

sys.path.insert(0, os.path.dirname(__file__))
import validatelib

def main():
    parser = argparse.ArgumentParser(
            parents=[
                validatelib.JobBuilder.get_general_validator_arg_parser(),
                ],
            )
    args = parser.parse_args()
    job_builder = validatelib.JobBuilder(
            nreps=args.nreps,
            random_seed=args.random_seed,
            job_queue=args.queue,
            virtual_env_path=args.venv,
            )

    ### PRODUCTION ###
    # num_focal_areas_regimes = [1, 8, ]
    # num_supplemental_areas_regimes = [0, 1, ]
    # birth_rate_regimes = [0.01, 0.05, 0.1,]
    # termination_condition_regime_types = ["target_focal_area_lineages"]
    # termination_condition_regime_values = [200,]

    # num_focal_areas_regimes = [1, ]
    # num_supplemental_areas_regimes = [0, ]
    # birth_rate_regimes = [0.1,]
    # termination_condition_regime_types = ["target_focal_area_lineages"]
    # termination_condition_regime_values = [50,]

    # ### DEVELOPMENT ###
    num_focal_areas_regimes = [1,]
    num_supplemental_areas_regimes = [0,]
    trait_nstates = [2, 8]
    trait_rate_regimes = []
    for f in (0.001, 0.010, 0.1):
        trait_rate_regimes.extend( [f * i for i in range(1, 10)] )
    birth_rate_regimes = [0.001, 0.010, 0.100]
    termination_condition_regime_types = ["target_focal_area_lineages"]
    termination_condition_regime_values = [50, 300]

    regime_sources = [
            num_focal_areas_regimes,
            num_supplemental_areas_regimes,
            trait_nstates,
            trait_rate_regimes,
            birth_rate_regimes,
            termination_condition_regime_types,
            termination_condition_regime_values,
            ]
    regimes = itertools.product(*regime_sources)

    for regime_idx, regime in enumerate(regimes):
        output_prefix = "r{:05d}".format(regime_idx+1)
        num_focal_areas = regime[0]
        num_supplemental_areas = regime[1]
        traits = [
            ("nst{}.q{}".format(regime[2], regime[3]), regime[2], regime[3]),
        ]
        birth_rate = regime[4]
        death_rate = 0.0
        dispersal_weight = birth_rate
        termination_condition_type = regime[5]
        termination_condition_value = regime[6]

        simulator_model_kwargs = {
                "num_focal_areas": num_focal_areas,
                "num_supplemental_areas": num_supplemental_areas,
                "lineage_birth_rate": ("fixed_value", birth_rate, birth_rate),
                "lineage_death_rate": ("fixed_value", death_rate, death_rate),
                "lineage_dispersal_weight": ("fixed_value", dispersal_weight, dispersal_weight),
                "global_dispersal_rate": 0,
                "termination_condition_type": termination_condition_type,
                "termination_condition_value": termination_condition_value,
                "traits": traits,
        }
        profiler_options = [
                    "--no-estimate-area-transition",
                ]
        job_builder.create_job(
                simulator_model_kwargs=simulator_model_kwargs,
                profiler_options=profiler_options,
                output_prefix=output_prefix,
                )

if __name__ == "__main__":
    main()

